==> babel-plugin-reflow-0.3.1.tgz: package/src/plugin/converters/type-annotation.ts <==
import {
  GenericTypeAnnotation,
  TSIndexedAccessType,
  TSTypeAnnotation,
  TSTypeLiteral,
  TSTypeOperator,
  TSTypeQuery,
  TSTypeReference,
  isIdentifier,
  tsTypeAnnotation,
  tsTypeReference,
  TypeAnnotation,
  FunctionTypeParam,
} from '@babel/types';
import { NodePath } from '@babel/traverse';

import { ConverterState } from '../types';
import { logWarning } from '../util/warnings';
import {
  convertCallUtility,
  convertClassUtil,
  convertDiffUtil,
  convertElementTypeUtil,
  convertExactUtil,
  convertKeysUtil,
  convertNonMaybeTypeUtil,
  convertPropertyTypeUtil,
  convertReadOnlyArrayUtil,
  convertRestUtil,
  convertShapeUtil,
} from './utility';
import { replaceNonPrimitiveType } from '../optimizers/non-primitive-types';
import { convertFlowType } from './flow-type';
import { convertIdentifier } from './identifier';
import { convertTypeParameterInstantiation } from './type-parameter';

type TSGenericTypeAnnotation =
  | TSIndexedAccessType
  | TSTypeLiteral
  | TSTypeOperator
  | TSTypeQuery
  | TSTypeReference;

export function convertTypeAnnotation(
  node: FunctionTypeParam | TypeAnnotation,
  state: ConverterState,
): TSTypeAnnotation {
  return tsTypeAnnotation(convertFlowType(node.typeAnnotation, state));
}

export function convertGenericTypeAnnotation(
  node: GenericTypeAnnotation,
  state: ConverterState,
  path: NodePath<GenericTypeAnnotation>,
): TSGenericTypeAnnotation {
  const id = convertIdentifier(node.id);

  const typeParameters = node.typeParameters
    ? convertTypeParameterInstantiation(node.typeParameters, state)
    : null;

  if (isIdentifier(id)) {
    if (typeParameters) {
      switch (id.name) {
        case '$Call':
          return convertCallUtility(typeParameters);

        case '$Diff':
          return convertDiffUtil(typeParameters);

        case '$ElementType':
          return convertElementTypeUtil(typeParameters);

        case '$Exact':
          return convertExactUtil(typeParameters);

        case '$Keys':
          return convertKeysUtil(typeParameters);

        case '$NonMaybeType':
          return convertNonMaybeTypeUtil(typeParameters);

        case '$PropertyType':
          return convertPropertyTypeUtil(typeParameters);

        case '$ReadOnlyArray':
          return convertReadOnlyArrayUtil(typeParameters);

        case '$Rest':
          return convertRestUtil(typeParameters);

        case '$Shape':
          return convertShapeUtil(typeParameters);

        case 'Class':
          return convertClassUtil(typeParameters, path);

        // Unsupported utility types
        case '$ObjiMap':
        case '$ObjMap':
        case '$Subtype':
        case '$Supertype':
        case '$TupleMap': {
          logWarning(
            {
              message: `The utility type ${id.name} is not supported by Reflow and will be retained in the output. Please fix this manually.`,
            },
            state.file.code,
            node.loc,
          );
        }
      }
    }

    id.name = replaceNonPrimitiveType(id.name);
  }

  return tsTypeReference(id, typeParameters);
}

==> babel-plugin-reflow-0.3.4.tgz: package/src/plugin/converters/type-annotation.ts <==
import {
  FunctionTypeParam,
  GenericTypeAnnotation,
  isIdentifier,
  TSAnyKeyword,
  tsAnyKeyword,
  TSIndexedAccessType,
  TSTypeAnnotation,
  tsTypeAnnotation,
  TSTypeLiteral,
  TSTypeOperator,
  TSTypeQuery,
  tsTypeReference,
  TSTypeReference,
  TypeAnnotation,
} from '@babel/types';
import { NodePath } from '@babel/traverse';

import { ConverterState } from '../types';
import { logWarning } from '../util/warnings';
import {
  convertCallUtility,
  convertClassUtil,
  convertDiffUtil,
  convertElementTypeUtil,
  convertExactUtil,
  convertKeysUtil,
  convertNonMaybeTypeUtil,
  convertPropertyTypeUtil,
  convertReadOnlyArrayUtil,
  convertRestUtil,
  convertShapeUtil,
} from './utility';
import { replaceNonPrimitiveType } from '../optimizers/non-primitive-types';
import { convertFlowType } from './flow-type';
import { convertIdentifier } from './identifier';
import { convertTypeParameterInstantiation } from './type-parameter';

type TSGenericTypeAnnotation =
  | TSAnyKeyword
  | TSIndexedAccessType
  | TSTypeLiteral
  | TSTypeOperator
  | TSTypeQuery
  | TSTypeReference;

export function convertTypeAnnotation(
  node: FunctionTypeParam | TypeAnnotation,
  state: ConverterState,
): TSTypeAnnotation {
  return tsTypeAnnotation(convertFlowType(node.typeAnnotation, state));
}

export function convertGenericTypeAnnotation(
  node: GenericTypeAnnotation,
  state: ConverterState,
  path: NodePath<GenericTypeAnnotation>,
): TSGenericTypeAnnotation {
  const id = convertIdentifier(node.id);

  const typeParameters = node.typeParameters
    ? convertTypeParameterInstantiation(node.typeParameters, state)
    : null;

  if (isIdentifier(id)) {
    if (typeParameters) {
      switch (id.name) {
        case '$Call':
          return convertCallUtility(typeParameters);

        case '$Diff':
          return convertDiffUtil(typeParameters);

        case '$ElementType':
          return convertElementTypeUtil(typeParameters);

        case '$Exact':
          return convertExactUtil(typeParameters);

        case '$Keys':
          return convertKeysUtil(typeParameters);

        case '$NonMaybeType':
          return convertNonMaybeTypeUtil(typeParameters);

        case '$PropertyType':
          return convertPropertyTypeUtil(typeParameters);

        case '$ReadOnlyArray':
          return convertReadOnlyArrayUtil(typeParameters);

        case '$Rest':
          return convertRestUtil(typeParameters);

        case '$Shape':
          return convertShapeUtil(typeParameters);

        case 'Class':
          return convertClassUtil(typeParameters, path);

        // Unsupported utility types
        case '$ObjMap':
        case '$ObjMapi':
        case '$Subtype':
        case '$Supertype':
        case '$TupleMap': {
          logWarning(
            {
              message: `The utility type ${id.name} is not supported by Reflow and will be replaced by 'any'. Please fix this type manually afterwards.`,
            },
            state.file.code,
            node.loc,
          );

          return tsAnyKeyword();
        }
      }
    }

    id.name = replaceNonPrimitiveType(id.name);
  }

  return tsTypeReference(id, typeParameters);
}
